package digital.laboratory.platform.reagent.controller;

import cn.hutool.core.io.file.FileNameUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.entity.InstructionBook;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.service.InstructionBookService;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import digital.laboratory.platform.reagent.vo.InstructionBookVO;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;


/**
 * (标准物质期间核查指导书)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质期间核查指导书) 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/instruction_book")
@Api(value = "instruction_book", tags = "(标准物质期间核查指导书)管理")
public class InstructionBookController {

    private final ReagentConsumablesService reagentConsumablesService;
    private final InstructionBookService instructionBookService;

    private final OssFile ossFile;

    /**
     * 新增(标准物质期间核查指导书)
     *
     * @param id (标准物质期间核查指导书)
     * @return R
     */
    @ApiOperation(value = "上传(标准物质期间核查指导书)", notes = "上传(标准物质期间核查指导书)")
    @SysLog("上传(标准物质期间核查指导书)")
    @PostMapping
//    @PreAuthorize("@pms.hasPermission('reagent_instruction_book_upload')" )
    public R<InstructionBook> upload(String id, @RequestPart(value = "file", required = false) MultipartFile file, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ReagentConsumables byId = reagentConsumablesService.getById(id);

        String path = "reagen_managment" + "/" + "instructionBook" + "/" + byId.getReagentConsumableId();
        String fileName = FileNameUtil.getName(file.getOriginalFilename());
        boolean b = ossFile.fileUpload(file, path);
        Map<String, String> ResultData = new HashMap<>();
        ResultData.put("fileName", fileName);
        ResultData.put("path", path);
        InstructionBook instructionBook = new InstructionBook();

        if (b) {
            instructionBook.setId(IdWorker.get32UUID().toUpperCase());
            instructionBook.setCommitTime(LocalDateTime.now());
            instructionBook.setMakerId(dlpUser.getId());
            instructionBook.setReferenceMaterialTypeId(byId.getReagentConsumableId());
            instructionBook.setInstructionBook(fileName);
            instructionBook.setCommitStatus(1);

            String fileNames = path + "/" + fileName;

            ossFile.fileGet(fileNames, httpServletResponse.getOutputStream());

            instructionBookService.save(instructionBook);

            return R.ok(instructionBook, "上传成功");
        } else return R.failed("上传失败");


    }

    /**
     * 新增(标准物质期间核查指导书)
     *
     * @param id (标准物质期间核查指导书)
     * @return R
     */
    @ApiOperation(value = "预览(标准物质期间核查指导书)", notes = "预览(标准物质期间核查指导书)(标准物质期间核查指导书)")
    @SysLog("预览(标准物质期间核查指导书)(标准物质期间核查指导书)")
    @GetMapping("/download")
//    @PreAuthorize("@pms.hasPermission('reagent_instruction_book_download')" )
    public R<InstructionBook> download(String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ReagentConsumables byId = reagentConsumablesService.getById(id);

        LambdaQueryWrapper<InstructionBook> instructionBookLambdaQueryWrapper = new LambdaQueryWrapper<>();

        instructionBookLambdaQueryWrapper.eq(InstructionBook::getReferenceMaterialTypeId, byId.getReagentConsumableId());

        InstructionBook one = instructionBookService.getOne(instructionBookLambdaQueryWrapper);

        if (one != null) {

            String fileName = one.getInstructionBook();

            String path = "reagen_managment" + "/" + "instructionBook" + "/" + byId.getReagentConsumableId();

            String fileNames = path + "/" + fileName;

            ossFile.fileGet(fileNames, httpServletResponse.getOutputStream());

            return R.ok(one);
        } else
            return R.ok(null);
    }

    /**
     * 新增(标准物质期间核查指导书)
     *
     * @param id (标准物质期间核查指导书)
     * @return R
     */
    @ApiOperation(value = "查看(标准物质期间核查指导书)", notes = "查看(标准物质期间核查指导书)(标准物质期间核查指导书)")
    @SysLog("查看(标准物质期间核查指导书)(标准物质期间核查指导书)")
    @GetMapping()
//    @PreAuthorize("@pms.hasPermission('reagent_instruction_book_get')" )
    public R<InstructionBookVO> getVOById(String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        InstructionBookVO vo = instructionBookService.getVO(id);

        return R.ok(vo);

    }

    /**
     * 新增(标准物质期间核查指导书)
     *
     * @param auditAndApproveDTO (标准物质期间核查指导书)
     * @return R
     */
    @ApiOperation(value = "审核(标准物质期间核查指导书)", notes = "审核(标准物质期间核查指导书)(标准物质期间核查指导书)")
    @SysLog("审核(标准物质期间核查指导书)(标准物质期间核查指导书)")
    @PostMapping("/audit")
    @PreAuthorize("@pms.hasPermission('reagent_instruction_book_audit')" )
    public R<InstructionBookVO> auditById(AuditAndApproveDTO auditAndApproveDTO , HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        InstructionBook instructionBook = instructionBookService.auditById(auditAndApproveDTO, dlpUser);

        InstructionBookVO vo = instructionBookService.getVO(instructionBook.getReferenceMaterialTypeId());

        return R.ok(vo);

    }

}





