package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.RequisitionRecord;
import digital.laboratory.platform.reagent.service.RequisitionRecordService;
import digital.laboratory.platform.reagent.vo.RequisitionRecordVO;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (试剂耗材领用记录表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材领用记录表) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/requisition_record" )
@Api(value = "requisition_record", tags = "(试剂耗材领用记录表)管理")
public class RequisitionRecordController {

    private final  RequisitionRecordService requisitionRecordService;


	/**
     * 分页查询
     * @param page 分页对象
     * @param requisitionRecord (试剂耗材领用记录表)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
//    @PreAuthorize("@pms.hasPermission('reagent_requisition_record_get')" )
    public R<IPage<RequisitionRecordVO>> getRequisitionRecordPage(Page<RequisitionRecord> page, RequisitionRecord requisitionRecord, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<RequisitionRecordVO> requisitionRecordSList = requisitionRecordService.getRequisitionRecordVOPage(page, Wrappers.<RequisitionRecord>query()
				.orderByDesc("create_time")
		);
		return R.ok(requisitionRecordSList);
//		return R.ok(requisitionRecordService.page(page, Wrappers.query(requisitionRecord)));
    }


//    /**
//     * 新增(试剂耗材领用记录表)
//     * @param requisitionRecord (试剂耗材领用记录表)
//     * @return R
//     */
//    @ApiOperation(value = "新增(试剂耗材领用记录表)", notes = "新增(试剂耗材领用记录表)")
//    @SysLog("新增(试剂耗材领用记录表)" )
//    @PostMapping
//    @PreAuthorize("@pms.hasPermission('reagent_requisition_record_add')" )
//    public R<RequisitionRecord> postAddObject(@RequestBody RequisitionRecord requisitionRecord, HttpServletRequest theHttpServletRequest) {
//		Principal principal = theHttpServletRequest.getUserPrincipal();
//		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
//
//        requisitionRecord.setRequisitionRecordId(IdWorker.get32UUID().toUpperCase());
//		if (requisitionRecordService.save(requisitionRecord)) {
//			return R.ok(requisitionRecord, "对象创建成功");
//		}
//		else {
//			return R.failed(requisitionRecord, "对象创建失败");
//		}
//    }
//
//    /**
//     * 修改(试剂耗材领用记录表)
//     * @param requisitionRecord (试剂耗材领用记录表)
//     * @return R
//     */
//    @ApiOperation(value = "修改(试剂耗材领用记录表)", notes = "修改(试剂耗材领用记录表)")
//    @SysLog("修改(试剂耗材领用记录表)" )
//    @PutMapping
//    @PreAuthorize("@pms.hasPermission('reagent_requisition_record_edit')" )
//    public R<RequisitionRecord> putUpdateById(@RequestBody RequisitionRecord requisitionRecord, HttpServletRequest theHttpServletRequest) {
//		Principal principal = theHttpServletRequest.getUserPrincipal();
//		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
//
//		if (requisitionRecordService.updateById(requisitionRecord)) {
//			return R.ok(requisitionRecord, "保存对象成功");
//		}
//		else {
//			return R.failed(requisitionRecord, "保存对象失败");
//		}
//    }
//
//    /**
//     * 通过id删除(试剂耗材领用记录表)
//     * @param requisitionRecordId id
//     * @return R
//     */
//    @ApiOperation(value = "通过id删除(试剂耗材领用记录表)", notes = "通过id删除(试剂耗材领用记录表)")
//    @SysLog("通过id删除(试剂耗材领用记录表)" )
//    @DeleteMapping("/{requisitionRecordId}" )
//    @PreAuthorize("@pms.hasPermission('reagent_requisition_record_del')" )
//    public R<RequisitionRecord> deleteById(@PathVariable String requisitionRecordId, HttpServletRequest theHttpServletRequest) {
//		Principal principal = theHttpServletRequest.getUserPrincipal();
//		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
//
//        RequisitionRecord oldRequisitionRecord = requisitionRecordService.getById(requisitionRecordId);
//
//		if (requisitionRecordService.removeById(requisitionRecordId)) {
//			return R.ok(oldRequisitionRecord, "对象删除成功");
//		}
//		else {
//			return R.failed(oldRequisitionRecord, "对象删除失败");
//		}
//
//    }

}
