package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.config.PageUtils;
import digital.laboratory.platform.reagent.dto.AcceptanceRecordFormDTO;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.entity.AcceptanceRecordForm;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.mapper.AcceptanceRecordFormMapper;
import digital.laboratory.platform.reagent.service.AcceptanceRecordFormService;
import digital.laboratory.platform.reagent.service.BlacklistService;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import digital.laboratory.platform.reagent.vo.AcceptanceRecordFormVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * (验收记录表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (验收记录表) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class AcceptanceRecordFormServiceImpl extends ServiceImpl<AcceptanceRecordFormMapper, AcceptanceRecordForm> implements AcceptanceRecordFormService {

    @Autowired
    private AcceptanceRecordFormService acceptanceRecordFormService;

    @Autowired
    private BlacklistService blacklistService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Override
    public AcceptanceRecordForm commitForm(AcceptanceRecordFormDTO acceptanceRecordFormDTO,DLPUser dlpUser) {

        AcceptanceRecordForm byId = acceptanceRecordFormService.getById(acceptanceRecordFormDTO.getAcceptanceRecordFormId());

        if (byId.getStatus()!=0){
            throw new RuntimeException(String.format("当前状态不能提交"));
        }
        BeanUtils.copyProperties(acceptanceRecordFormDTO, byId);

        byId.setUserName(dlpUser.getName());

        byId.setCommitTime(LocalDateTime.now());

        byId.setStatus(1);
        if (acceptanceRecordFormService.updateById(byId)) {
            return byId;
        } else {
            throw new RuntimeException(String.format("保存失败"));
        }
    }

    @Override
    public AcceptanceRecordForm addForm(String reagentConsumableId, String supplierId) {

        AcceptanceRecordForm acceptanceRecordForm = new AcceptanceRecordForm();

        acceptanceRecordForm.setId(IdWorker.get32UUID().toUpperCase());

        acceptanceRecordForm.setSupplierId(supplierId);

        acceptanceRecordForm.setReagentConsumableId(reagentConsumableId);

        acceptanceRecordForm.setStatus(0);

        if ((acceptanceRecordFormService.save(acceptanceRecordForm))) {

            return acceptanceRecordForm;

        } else throw new RuntimeException(String.format("创建验收任务失败"));
    }

    @Override
    public IPage<AcceptanceRecordFormVO> getAcceptanceRecordFormVOPage(IPage<AcceptanceRecordForm> page) {

        IPage<AcceptanceRecordFormVO> acceptanceRecordFormVOPage = baseMapper.getAcceptanceRecordFormVOPage(page);

        List<AcceptanceRecordFormVO> records = acceptanceRecordFormVOPage.getRecords();

        for (AcceptanceRecordFormVO record : records) {

            ReagentConsumables byId = reagentConsumablesService.getById(record.getReagentConsumableId());

            record.setReagentConsumables(byId);
        }
        return acceptanceRecordFormVOPage;

    }

    @Override
    public IPage<AcceptanceRecordFormVO> getAcceptanceRecordFormVORecordPage(IPage<AcceptanceRecordForm> page) {

        LambdaQueryWrapper<AcceptanceRecordForm> acceptanceRecordFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

        acceptanceRecordFormLambdaQueryWrapper.eq(AcceptanceRecordForm::getStatus,4);

        List<AcceptanceRecordForm> list = acceptanceRecordFormService.list(acceptanceRecordFormLambdaQueryWrapper);

        ArrayList<AcceptanceRecordFormVO> acceptanceRecordFormVOS = new ArrayList<>();

        if (list.size()!=0){

        for (AcceptanceRecordForm acceptanceRecordForm : list) {

            AcceptanceRecordFormVO acceptanceRecordFormVO = acceptanceRecordFormService.getAcceptanceRecordFormVO(acceptanceRecordForm.getId());

            acceptanceRecordFormVOS.add(acceptanceRecordFormVO);
        }}
        PageUtils pageUtils = new PageUtils();

        Page pages = pageUtils.getPages((int) page.getPages(), (int) page.getSize(), acceptanceRecordFormVOS);

        return pages;
    }

    @Override
    public AcceptanceRecordFormVO getAcceptanceRecordFormVO(String acceptanceRecordFormId) {

        AcceptanceRecordFormVO acceptanceRecordFormVO = baseMapper.getAcceptanceRecordFormVO(acceptanceRecordFormId);

        ReagentConsumables byId = reagentConsumablesService.getById(acceptanceRecordFormVO.getReagentConsumableId());

        acceptanceRecordFormVO.setReagentConsumables(byId);

        return acceptanceRecordFormVO;
    }

    @Override//一级审核
    public AcceptanceRecordForm primaryAudit(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        AcceptanceRecordForm byId = acceptanceRecordFormService.getById(auditAndApproveDTO.getUuId());

        if (byId.getStatus()!=1){
            throw new RuntimeException(String.format("当前状态不能审核"));
        }
        byId.setPrimaryAuditorId(dlpUser.getId());
        byId.setAuditOpinionOfPrimary(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditTimeOfPrimary(LocalDateTime.now());
        byId.setAuditResultOfPrimary(auditAndApproveDTO.getAuditResult());

        if (auditAndApproveDTO.getAuditResult()==true){
            byId.setStatus(2);
        }else byId.setStatus(-2);


        if (acceptanceRecordFormService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("审核失败"));
    }
    @Override//二级审核
    public AcceptanceRecordForm secondaryAudit(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        AcceptanceRecordForm byId = acceptanceRecordFormService.getById(auditAndApproveDTO.getUuId());

        if (byId.getStatus()!=2){
            throw new RuntimeException(String.format("当前状态不能审核"));
        }
        byId.setSecondaryAuditorId(dlpUser.getId());
        byId.setAuditOpinionOfSecondary(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditTimeOfSecondary(LocalDateTime.now());
        byId.setAuditResultOfSecondary(auditAndApproveDTO.getAuditResult());

        if (auditAndApproveDTO.getAuditResult()==true){
            byId.setStatus(3);
        }else byId.setStatus(-3);

        if (acceptanceRecordFormService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("审核失败"));
    }
    @Override//三级审核
    public AcceptanceRecordForm threeLevelAudit(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        AcceptanceRecordForm byId = acceptanceRecordFormService.getById(auditAndApproveDTO.getUuId());

        if (byId.getStatus()!=3){
            throw new RuntimeException(String.format("当前状态不能审核"));
        }
        byId.setThreeLevelAuditorId(dlpUser.getId());
        byId.setAuditOpinionOfThreeLevel(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditTimeOfThreeLevel(LocalDateTime.now());
        byId.setAuditResultOfThreeLevel(auditAndApproveDTO.getAuditResult());

        if (auditAndApproveDTO.getAuditResult()==true){

            if (byId.getNonconformingItem()!=null){

                blacklistService.addListById2(byId.getReagentConsumableId(),byId.getSupplierId());
            }
            byId.setStatus(4);
        }else byId.setStatus(-4);

        if (acceptanceRecordFormService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("审核失败"));
    }


}
