package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.DateTime;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.*;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.DecentralizedRequestMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.DecentralizedRequestVO;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * (分散采购申请)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (分散采购申请) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class DecentralizedRequestServiceImpl extends ServiceImpl<DecentralizedRequestMapper, DecentralizedRequest> implements DecentralizedRequestService {

    @Autowired
    private DecentralizedRequestService decentralizedRequestService;
    @Autowired
    private DecentralizeDetailsService decentralizeDetailsService;

    @Autowired
    private ComplianceCheckService complianceCheckService;
    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private PurchaseListService purchaseListService;

    @Override//分散采购申请分页
    public IPage<DecentralizedRequestVO> getDecentralizedRequestVOPage
            (@Param("page") IPage<DecentralizedRequest> page, @Param(Constants.WRAPPER) QueryWrapper<DecentralizedRequest> qw) {

        IPage<DecentralizedRequestVO> decentralizedRequestVOPage = baseMapper.getDecentralizedRequestVOPage(page, qw);

        return decentralizedRequestVOPage;
    }


    @Override//分散采购申请列表
    public List<DecentralizedRequestVO> getDecentralizedRequestVOList(@Param(Constants.WRAPPER) QueryWrapper<DecentralizedRequest> qw) {

        List<DecentralizedRequestVO> decentralizedRequestVOList = baseMapper.getDecentralizedRequestVOList(qw);

        return decentralizedRequestVOList;
    }

    @Override//通过ID查询分散采购申请
    public DecentralizedRequestVO getRequestById(String decentralizedRequestId) {

        DecentralizedRequestVO decentralizedRequestVO = baseMapper.getDecentralizedRequest(decentralizedRequestId);

        List<DecentralizeDetails> decentralizeDetailsList = decentralizeDetailsService.getDecentralizeDetailsList(decentralizedRequestId);

        decentralizedRequestVO.setDecentralizeDetailsVOList(decentralizeDetailsList);

        return decentralizedRequestVO;
    }

    @Override//创建分散采购申请
    public DecentralizedRequest addRequest(List<DecentralizedRequestDTO> decentralizedRequestDTOList, DLPUser dlpUser) {

        DecentralizedRequest decentralizedRequest = new DecentralizedRequest();

        decentralizedRequest.setId(IdWorker.get32UUID().toUpperCase());

        decentralizedRequest.setApplicantId(dlpUser.getId());

        decentralizedRequest.setDateOfApplication(LocalDateTime.now());

        decentralizedRequest.setStatus(0);

        List<DecentralizeDetails> list = new ArrayList<>();
        //接收分散采购DTO，将其赋值给分散采购明细
        for (DecentralizedRequestDTO decentralizedRequestDTO : decentralizedRequestDTOList) {
            //新添加试剂耗材

            DecentralizeDetails decentralizedDetails = new DecentralizeDetails();

            BeanUtils.copyProperties(decentralizedRequestDTO, decentralizedDetails);

            decentralizedDetails.setId(IdWorker.get32UUID().toUpperCase());

            decentralizedDetails.setDecentralizedRequestId(decentralizedRequest.getId());

            list.add(decentralizedDetails);
        }
        if (decentralizedRequestService.save(decentralizedRequest) & decentralizeDetailsService.saveBatch(list)) {
            return decentralizedRequest;
        } else return null;
    }

    @Override
    public DecentralizeDetails addDetails(DecentralizedRequestDTO decentralizedRequestDTO) {

        DecentralizeDetails decentralizeDetails = new DecentralizeDetails();

        BeanUtils.copyProperties(decentralizedRequestDTO, decentralizeDetails);

        if (decentralizeDetailsService.save(decentralizeDetails)) {
            return decentralizeDetails;
        } else throw new RuntimeException(String.format("保存失败"));
    }

    @Override//修改分散采购申请明细
    public DecentralizeDetails editRequest(DecentralizedRequestDTO decentralizedRequestDTO) {

        DecentralizeDetails decentralizeDetails = decentralizeDetailsService.getById(decentralizedRequestDTO.getDecentralizeDetailsId());

        BeanUtils.copyProperties(decentralizedRequestDTO, decentralizeDetails);

        if (decentralizeDetailsService.updateById(decentralizeDetails)) {
            return decentralizeDetails;
        } else return null;

    }

    @Override//删除分散采购申请
    public Boolean delRequestById(String decentralizedRequestId) {

        LambdaQueryWrapper<DecentralizeDetails> decentralizeDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

        decentralizeDetailsLambdaQueryWrapper.eq(DecentralizeDetails::getDecentralizedRequestId, decentralizedRequestId);

        List<DecentralizeDetails> list = decentralizeDetailsService.list(decentralizeDetailsLambdaQueryWrapper);

        if (list != null) {
            decentralizeDetailsService.removeBatchByIds(list);
        }

        return decentralizedRequestService.removeById(decentralizedRequestService.getById(decentralizedRequestId));

    }

    @Override//提交分散采购申请
    public DecentralizedRequest commitRequest(List<DecentralizedRequestDTO> decentralizedRequestDTOList, DLPUser dlpUser) {
        String decentralizedRequestId = decentralizedRequestDTOList.get(0).getDecentralizedRequestId();

        if (decentralizedRequestId == null) {

            DecentralizedRequest decentralizedRequest = decentralizedRequestService.addRequest(decentralizedRequestDTOList, dlpUser);

            decentralizedRequest.setStatus(1);

            decentralizedRequest.setCommitTime(LocalDateTime.now());

            if (decentralizedRequestService.updateById(decentralizedRequest)) {
                return decentralizedRequest;
            } else throw new RuntimeException(String.format("提交失败"));
        } else {

            DecentralizedRequest byId = decentralizedRequestService.getById(decentralizedRequestId);

            byId.setStatus(1);

            byId.setCommitTime(LocalDateTime.now());

            if (decentralizedRequestService.updateById(byId)) {
                return byId;
            } else throw new RuntimeException(String.format("提交失败"));
        }

    }

    @Override//一级审核
    public DecentralizedRequest primaryAuditRequest(DLPUser dlpUser, AuditAndApproveDTO auditAndApproveDto) {

        DecentralizedRequest byId = decentralizedRequestService.getById(auditAndApproveDto.getUuId());

        if (byId.getStatus()!=1){
           throw new RuntimeException(String.format("当前状态不能审核"));
        }

        byId.setPrimaryAuditorId(dlpUser.getId());

        byId.setAuditResultOfPrimary(auditAndApproveDto.getAuditResult());

        byId.setAuditOpinionOfPrimary(auditAndApproveDto.getAuditOpinion());

        byId.setAuditTimeOfPrimary(LocalDateTime.now());

        if (auditAndApproveDto.getAuditResult() == true) {
            byId.setStatus(2);
        } else {
            byId.setStatus(-2);
        }

        if (decentralizedRequestService.updateById(byId)) {
            return byId;
        } else return null;


    }

    @Override//二级审核
    public DecentralizedRequest secondaryAuditRequest(DLPUser dlpUser, AuditAndApproveDTO auditAndApproveDto) {

        DecentralizedRequest byId = decentralizedRequestService.getById(auditAndApproveDto.getUuId());

        if (byId.getStatus()!=2){
            throw new RuntimeException(String.format("当前状态不能审核"));
        }

        byId.setSecondaryAuditorId(dlpUser.getId());

        byId.setAuditResultOfSecondary(auditAndApproveDto.getAuditResult());

        byId.setAuditOpinionOfSecondary(auditAndApproveDto.getAuditOpinion());

        byId.setAuditTimeOfSecondary(LocalDateTime.now());

        if (auditAndApproveDto.getAuditResult() == true) {
            byId.setStatus(3);
        } else {
            byId.setStatus(-3);
        }

        if (decentralizedRequestService.updateById(byId)) {
            return byId;
        } else return null;
    }

    @Transactional
    @Override//三级审核
    public DecentralizedRequest threeLevelAuditRequest(DLPUser dlpUser, AuditAndApproveDTO auditAndApproveDTO) {

        DecentralizedRequest byId = decentralizedRequestService.getById(auditAndApproveDTO.getUuId());

        if (byId.getStatus()!=3){
            throw new RuntimeException(String.format("当前状态不能审核"));
        }

        byId.setThreeLevelAuditId(dlpUser.getId());

        byId.setAuditResultOfThreeLevel(auditAndApproveDTO.getAuditResult());

        byId.setAuditOpinionOfThreeLevel(auditAndApproveDTO.getAuditOpinion());

        byId.setAuditTimeOfThreeLevel(LocalDateTime.now());

        if (auditAndApproveDTO.getAuditResult() == true) {
            byId.setStatus(4);
        } else {

            byId.setStatus(-4);
        }
        if (decentralizedRequestService.updateById(byId)) {
            return byId;
        } else return null;
    }

    @Override//审批
    @Transactional
    public DecentralizedRequest approveRequest(DLPUser dlpUser, AuditAndApproveDTO auditAndApproveDto) {

        DecentralizedRequest byId = decentralizedRequestService.getById(auditAndApproveDto.getUuId());

        if (byId.getStatus()!=4){
            throw new RuntimeException(String.format("当前状态不能审批"));
        }
            byId.setApproverId(dlpUser.getId());

            byId.setResultOfApproval(auditAndApproveDto.getApproveResult());

            byId.setOpinionOfApproval(auditAndApproveDto.getApproveOpinion());

            byId.setApprovalOfTime(LocalDateTime.now());

        if (auditAndApproveDto.getApproveResult() == true) {
            //创建采购清单
            LambdaQueryWrapper<DecentralizeDetails> decentralizeDetailsLambdaQueryWrapper1 = new LambdaQueryWrapper<>();

            decentralizeDetailsLambdaQueryWrapper1.eq(DecentralizeDetails::getDecentralizedRequestId, byId.getId());

            List<DecentralizeDetails> list = decentralizeDetailsService.list(decentralizeDetailsLambdaQueryWrapper1);

            PurchaseListDTO purchaseListDTO = new PurchaseListDTO();

            PurchaseList purchaseList = new PurchaseList();
            purchaseList.setId(IdWorker.get32UUID().toUpperCase());
            purchaseList.setName(DateTime.now() + "采购清单");
            purchaseList.setStatus(0);
            purchaseList.setType("分散采购申请");

            purchaseListService.save(purchaseList);

            for (DecentralizeDetails decentralizeDetails : list) {

                purchaseListDTO.setReagentConsumableId(decentralizeDetails.getReagentConsumableId());
                purchaseListDTO.setPurchaseQuantity(decentralizeDetails.getQuantity());
                purchaseListDTO.setPurchaseListId(purchaseList.getId());

                purchaseListService.addDetails(purchaseListDTO);
            }


            if (byId.getResultOfApproval() == true) {
                //如果未购入过该物品，则将该物品信息更新至试剂耗材类
                LambdaQueryWrapper<DecentralizeDetails> decentralizeDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

                decentralizeDetailsLambdaQueryWrapper.eq(DecentralizeDetails::getDecentralizedRequestId, byId.getId());

                List<DecentralizeDetails> decentralizeDetailsList = decentralizeDetailsService.list(decentralizeDetailsLambdaQueryWrapper);

                LambdaQueryWrapper<ReagentConsumables> reagentConsumablesLambdaQueryWrapper = new LambdaQueryWrapper<>();

                for (DecentralizeDetails decentralizeDetails : decentralizeDetailsList) {

                    if (decentralizeDetails.getReagentConsumableId() == null) {

                        ReagentConsumables reagentConsumables = reagentConsumablesService.addReagentConsumables(decentralizeDetails);

                        decentralizeDetails.setReagentConsumableId(reagentConsumables.getReagentConsumableId());

                        decentralizeDetailsService.updateById(decentralizeDetails);
                    }
                }
            }

            byId.setStatus(5);
        } else {
            byId.setStatus(-5);
        }
        if (decentralizedRequestService.updateById(byId)) {
            return byId;
        } else return null;

    }
}




