package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.EvaluationFormDTO;
import digital.laboratory.platform.reagent.entity.EvaluationForm;
import digital.laboratory.platform.reagent.mapper.EvaluationFormMapper;
import digital.laboratory.platform.reagent.service.EvaluationFormService;
import digital.laboratory.platform.reagent.service.ProvideServicesOrSuppliesService;
import digital.laboratory.platform.reagent.service.PurchaseListDetailsService;
import digital.laboratory.platform.reagent.service.SupplierInformationService;
import digital.laboratory.platform.reagent.vo.EvaluationFormVO;
import digital.laboratory.platform.reagent.vo.ProvideServicesOrSuppliesVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * (服务商/供应商评价表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (服务商 / 供应商评价表) 服务实现类
 */
@SuppressWarnings("all")
@Service
public class EvaluationFormServiceImpl extends ServiceImpl<EvaluationFormMapper, EvaluationForm> implements EvaluationFormService {

    @Autowired
    private EvaluationFormService evaluationFormService;
    @Autowired
    private ProvideServicesOrSuppliesService provideServicesOrSuppliesService;
    @Autowired
    private SupplierInformationService supplierInformationService;
    @Autowired
    private PurchaseListDetailsService purchaseListDetailsService;

    @Override//通过供应商ID查询评价表列表
    public List<EvaluationFormVO> getEvaluationFormVOList(String supplierInformationId) {

        List<EvaluationFormVO> evaluationFormVOList = baseMapper.getEvaluationFormVOList(supplierInformationId);

        for (EvaluationFormVO evaluationFormVO : evaluationFormVOList) {

            List<ProvideServicesOrSuppliesVO> voList = provideServicesOrSuppliesService.getVOList(evaluationFormVO.getId());

            evaluationFormVO.setProvideServicesOrSuppliesVOList(voList);
        }

        return evaluationFormVOList;
    }

    @Override
    public EvaluationFormVO getFormById(String evaluationFormId) {

        EvaluationFormVO evaluationForm = baseMapper.getEvaluationForm(evaluationFormId);

        List<ProvideServicesOrSuppliesVO> voList = provideServicesOrSuppliesService.getVOList(evaluationFormId);

        evaluationForm.setProvideServicesOrSuppliesVOList(voList);

        return evaluationForm;
    }

    @Override
    public EvaluationForm addForm(String supplierInformationId) {

        EvaluationForm evaluationForm = new EvaluationForm();
        evaluationForm.setId(IdWorker.get32UUID().toUpperCase());
        evaluationForm.setSupplierInformationId(supplierInformationId);

        if (evaluationFormService.save(evaluationForm)) {
            return evaluationForm;
        } else return null;

    }


    @Override//填写评价表并提交
    public EvaluationForm editFormById(EvaluationFormDTO evaluationFormDTO, DLPUser dlpUser) {

        if (!(StrUtil.isNotBlank(evaluationFormDTO.getContactNumber()) &
                StrUtil.isNotBlank(evaluationFormDTO.getContactPerson()) &
                StrUtil.isNotBlank(evaluationFormDTO.getSupplierBusinessLicense())&
                StrUtil.isNotBlank(evaluationFormDTO.getSupplierPassesQualityAssuranceSystem())&
                StrUtil.isNotBlank(evaluationFormDTO.getSupplierProductCertification())&
                StrUtil.isNotBlank(evaluationFormDTO.getCheckAndCalibrateEfficiencyOfSupplies())&
                StrUtil.isNotBlank(evaluationFormDTO.getOverallSupplierServiceSatisfaction())&
                StrUtil.isNotBlank(evaluationFormDTO.getSupplierAttitude())&
                StrUtil.isNotBlank(evaluationFormDTO.getSupplierEquipmentAndFacilities())&
                StrUtil.isNotBlank(evaluationFormDTO.getSupplierTechnologyAndManagementCapability())&
                StrUtil.isNotBlank(evaluationFormDTO.getWhetherTheSupplierDeliversOnTime())&
                StrUtil.isNotBlank(evaluationFormDTO.getSupplierInformationId())&
                StrUtil.isNotBlank(evaluationFormDTO.getEvaluationFormId()))){

            throw new RuntimeException(String.format("请完善评价信息后再提交"));
        }

        EvaluationForm evaluationForm = evaluationFormService.getById(evaluationFormDTO.getEvaluationFormId());

        BeanUtils.copyProperties(evaluationFormDTO, evaluationForm);

        evaluationForm.setPrimaryUserId(dlpUser.getId());

        evaluationForm.setCommentsDateFromPrimary(LocalDate.now());

        evaluationForm.setStatus(2);

        evaluationForm.setCommitTime(LocalDateTime.now());

        if (evaluationFormService.updateById(evaluationForm)) {
            return evaluationForm;
        } else throw new RuntimeException(String.format("提交失败"));
    }

    @Override//二级审核
    public EvaluationForm auditFormOfSecondary(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        EvaluationForm byId = evaluationFormService.getById(auditAndApproveDTO.getUuId());

        byId.setSecondaryUserId(dlpUser.getId());

        byId.setCommentsDateFromSecondary(LocalDate.now());

        byId.setCommentsFromSecondary(auditAndApproveDTO.getAuditOpinion());

        byId.setCommentsResultFromSecondary(auditAndApproveDTO.getAuditResult());

        if (auditAndApproveDTO.getAuditResult() == true) {
            byId.setStatus(3);
        } else byId.setStatus(-3);

        if (evaluationFormService.updateById(byId)) {
            return byId;
        } else {
            return null;
        }
    }

    @Override//三级审核

    public EvaluationForm auditFormOfThreeLevel(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        EvaluationForm byId = evaluationFormService.getById(auditAndApproveDTO.getUuId());

        byId.setThreeLevelUserId(dlpUser.getId());

        byId.setCommentsDateFromThreeLevel(LocalDate.now());

        byId.setCommentsFromThreeLevel(auditAndApproveDTO.getAuditOpinion());

        byId.setCommentsResultFromThreeLevel(auditAndApproveDTO.getAuditResult());

        if (auditAndApproveDTO.getAuditResult() == true) {
            byId.setStatus(4);
        } else byId.setStatus(-4);

        if (evaluationFormService.updateById(byId)) {
            return byId;
        } else {
            return null;
        }
    }

    @Override//通过供应商ID查询评价表列表
    public EvaluationFormVO getEvaluationFormVO(String id) {

        EvaluationFormVO evaluationForm = baseMapper.getEvaluationForm(id);


        List<ProvideServicesOrSuppliesVO> voList = provideServicesOrSuppliesService.getVOList(evaluationForm.getId());

        evaluationForm.setProvideServicesOrSuppliesVOList(voList);


        return evaluationForm;
    }


}









