package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.entity.InstructionBook;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.mapper.InstructionBookMapper;
import digital.laboratory.platform.reagent.service.InstructionBookService;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import digital.laboratory.platform.reagent.vo.InstructionBookVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * (标准物质期间核查指导书)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质期间核查指导书) 服务实现类
 */
@Service
public class InstructionBookServiceImpl extends ServiceImpl<InstructionBookMapper, InstructionBook> implements InstructionBookService {

    @Autowired
    private InstructionBookService instructionBookService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;
    @Override
    public InstructionBook auditById(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser){

        InstructionBook byId = instructionBookService.getById(auditAndApproveDTO.getUuId());

        byId.setAuditOpinionOfTechnical(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditResultOfTechnical(auditAndApproveDTO.getAuditResult());
        byId.setAuditTimeOfTechnical(LocalDateTime.now());
        byId.setTechnicalDirectorId(dlpUser.getId());
        byId.setCreateName(dlpUser.getName());

        if (auditAndApproveDTO.getAuditResult()==true){

            byId.setCommitStatus(2);
        }
        else byId.setCommitStatus(-2);

       if (instructionBookService.updateById(byId)){

           return byId;
       }else return null;
    }

    @Override
    public InstructionBookVO getVO(String id){

        LambdaQueryWrapper<InstructionBook> instructionBookLambdaQueryWrapper = new LambdaQueryWrapper<>();

        instructionBookLambdaQueryWrapper.eq(InstructionBook::getReferenceMaterialTypeId,id);

        InstructionBook one = instructionBookService.getOne(instructionBookLambdaQueryWrapper);

        InstructionBookVO instructionBookVO = new InstructionBookVO();

        BeanUtils.copyProperties(one,instructionBookVO);

        ReagentConsumables byId = reagentConsumablesService.getById(instructionBookVO.getReferenceMaterialTypeId());

        instructionBookVO.setReagentConsumables(byId);

        return instructionBookVO;
    }

}
