package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.ReviewAndApproveMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

@Service
@SuppressWarnings("all")
public class ReviewAndApproveServiceImpl extends ServiceImpl<ReviewAndApproveMapper, ReviewAndApprove> implements ReviewAndApproveService {

    @Autowired
    private DecentralizedRequestService decentralizedRequestService;

    @Autowired
    private PurchasingPlanService purchasingPlanService;

    @Autowired
    private AcceptanceRecordFormService acceptanceRecordFormService;

    @Autowired
    private ComplianceCheckService complianceCheckService;

    @Autowired
    private CheckScheduleService checkScheduleService;

    @Autowired
    private PeriodVerificationImplementationService periodVerificationImplementationService;

    @Autowired
    private PurchaseCatalogueService purchaseCatalogueService;

    @Autowired
    private StandardMaterialApprovalFormService standardMaterialApprovalFormService;

    @Autowired
    private EvaluationFormService evaluationFormService;

    @Autowired
    private InstructionBookService instructionBookService;

    @Override
    // 进行审核的列表
    public ReviewAndApprove getReviewAndApproveList(DLPUser dlpUser) {

        Set<String> permissions = dlpUser.getPermissions();

        ReviewAndApprove reviewAndApprove = new ReviewAndApprove();
        //分散采购申请待审核列表
        List<DecentralizedRequestVO> decentralizedRequestVOS = new ArrayList<>();
        //采购目录待审核列表
        List<PurchaseCatalogueVO> purchaseCatalogueVOList = new ArrayList<>();
        //采购计划待审核列表
        List<PurchasingPlanVO> purchasingPlanVOS = new ArrayList<>();
        //符合性检查待审核列表
        List<ComplianceCheckVO> complianceCheckVOS = new ArrayList<>();
        //指导书待审核列表
        List<InstructionBookVO> instructionBookVOS = new ArrayList<>();
        //期间核查待审核列表
        List<CheckScheduleVO> checkScheduleVOS = new ArrayList<>();
        //标准物质管理待审核列表
        List<StandardMaterialApprovalFormVO> standardMaterialApprovalFormVOS = new ArrayList<>();
        //验收记录待审核列表
        List<AcceptanceRecordFormVO> acceptanceRecordFormVOS = new ArrayList<>();
        //期间核查结果录入待审核列表
        List<PeriodVerificationImplementationVO> periodVerificationImplementationVOS = new ArrayList<>();
        //供应商评价待审核列表
        List<EvaluationFormVO> evaluationFormVOS = new ArrayList<>();


        if (permissions.contains("reagent_decentralized_request_primary")) {

            //分散采购申请一级审核列表
            LambdaQueryWrapper<DecentralizedRequest> decentralizedRequestLambdaQueryWrapper = new LambdaQueryWrapper<>();

            decentralizedRequestLambdaQueryWrapper.eq(DecentralizedRequest::getStatus, 1);

            List<DecentralizedRequest> decentralizedRequestList = decentralizedRequestService.list(decentralizedRequestLambdaQueryWrapper);

            List<DecentralizedRequestVO> decentralizedRequestVOList = new ArrayList<>();

            for (DecentralizedRequest decentralizedRequest : decentralizedRequestList) {

                DecentralizedRequestVO requestById = decentralizedRequestService.getRequestById(decentralizedRequest.getId());

                decentralizedRequestVOList.add(requestById);
            }
            decentralizedRequestVOS.addAll(decentralizedRequestVOList);
        }

        if (permissions.contains("reagent_decentralized_request_secondary")) {

            //分散采购申请二级审核列表
            LambdaQueryWrapper<DecentralizedRequest> decentralizedRequestLambdaQueryWrapper = new LambdaQueryWrapper<>();

            decentralizedRequestLambdaQueryWrapper.eq(DecentralizedRequest::getStatus, 2);

            List<DecentralizedRequest> decentralizedRequestList = decentralizedRequestService.list(decentralizedRequestLambdaQueryWrapper);

            List<DecentralizedRequestVO> decentralizedRequestVOList = new ArrayList<>();

            for (DecentralizedRequest decentralizedRequest : decentralizedRequestList) {

                DecentralizedRequestVO requestById = decentralizedRequestService.getRequestById(decentralizedRequest.getId());

                decentralizedRequestVOList.add(requestById);
            }
            decentralizedRequestVOS.addAll(decentralizedRequestVOList);
        }

        if (permissions.contains("reagent_decentralized_request_threeLevel")) {

            //分散采购申请三级审核列表
            LambdaQueryWrapper<DecentralizedRequest> decentralizedRequestLambdaQueryWrapper = new LambdaQueryWrapper<>();

            decentralizedRequestLambdaQueryWrapper.eq(DecentralizedRequest::getStatus, 3);

            List<DecentralizedRequest> decentralizedRequestList = decentralizedRequestService.list(decentralizedRequestLambdaQueryWrapper);

            List<DecentralizedRequestVO> decentralizedRequestVOList = new ArrayList<>();

            for (DecentralizedRequest decentralizedRequest : decentralizedRequestList) {

                DecentralizedRequestVO requestById = decentralizedRequestService.getRequestById(decentralizedRequest.getId());

                decentralizedRequestVOList.add(requestById);
            }
            decentralizedRequestVOS.addAll(decentralizedRequestVOList);
        }

        if (permissions.contains("reagent_decentralized_request_approve")) {
            //分散采购申请审批列表
            LambdaQueryWrapper<DecentralizedRequest> decentralizedRequestLambdaQueryWrapper = new LambdaQueryWrapper<>();

            decentralizedRequestLambdaQueryWrapper.eq(DecentralizedRequest::getStatus, 4);

            List<DecentralizedRequest> decentralizedRequestList = decentralizedRequestService.list(decentralizedRequestLambdaQueryWrapper);

            List<DecentralizedRequestVO> decentralizedRequestVOList = new ArrayList<>();

            for (DecentralizedRequest decentralizedRequest : decentralizedRequestList) {

                DecentralizedRequestVO requestById = decentralizedRequestService.getRequestById(decentralizedRequest.getId());

                decentralizedRequestVOList.add(requestById);
            }
            decentralizedRequestVOS.addAll(decentralizedRequestVOList);
        }

        if (permissions.contains("reagent_purchase_catalogue_primary")) {
            //采购目录一级审核列表
            LambdaQueryWrapper<PurchaseCatalogue> purchaseCatalogueLambdaQueryWrapper = new LambdaQueryWrapper<>();

            purchaseCatalogueLambdaQueryWrapper.eq(PurchaseCatalogue::getStatus, 1);

            List<PurchaseCatalogue> list = purchaseCatalogueService.list(purchaseCatalogueLambdaQueryWrapper);

            ArrayList<PurchaseCatalogueVO> purchaseCatalogueVOArrayList = new ArrayList<>();

            for (PurchaseCatalogue purchaseCatalogue : list) {

                PurchaseCatalogueVO purchaseCatalogueVO = purchaseCatalogueService.getPurchaseCatalogueVO(purchaseCatalogue.getPurchaseCatalogueId());

                purchaseCatalogueVOArrayList.add(purchaseCatalogueVO);
            }
            purchaseCatalogueVOList.addAll(purchaseCatalogueVOArrayList);

        }
        //采购目录二级审核
        if (permissions.contains("reagent_purchase_catalogue_secondary")) {

            LambdaQueryWrapper<PurchaseCatalogue> purchaseCatalogueLambdaQueryWrapper = new LambdaQueryWrapper<>();

            purchaseCatalogueLambdaQueryWrapper.eq(PurchaseCatalogue::getStatus, 2);

            List<PurchaseCatalogue> list = purchaseCatalogueService.list(purchaseCatalogueLambdaQueryWrapper);

            ArrayList<PurchaseCatalogueVO> purchaseCatalogueVOArrayList = new ArrayList<>();

            for (PurchaseCatalogue purchaseCatalogue : list) {

                PurchaseCatalogueVO purchaseCatalogueVO = purchaseCatalogueService.getPurchaseCatalogueVO(purchaseCatalogue.getPurchaseCatalogueId());

                purchaseCatalogueVOArrayList.add(purchaseCatalogueVO);

            }
            purchaseCatalogueVOList.addAll(purchaseCatalogueVOArrayList);

        }
        //采购计划审核
        if (permissions.contains("reagent_purchasing_plan_audit")) {

            LambdaQueryWrapper<PurchasingPlan> purchasingPlanLambdaQueryWrapper = new LambdaQueryWrapper<>();

            purchasingPlanLambdaQueryWrapper.eq(PurchasingPlan::getStatus, 1);

            List<PurchasingPlan> list = purchasingPlanService.list(purchasingPlanLambdaQueryWrapper);

            ArrayList<PurchasingPlanVO> purchasingPlanVOS1 = new ArrayList<>();

            for (PurchasingPlan purchasingPlan : list) {

                PurchasingPlanVO purchasingPlanVO = purchasingPlanService.getPurchasingPlanVO(purchasingPlan.getPurchasingPlanId());

                purchasingPlanVOS1.add(purchasingPlanVO);
            }
            purchasingPlanVOS.addAll(purchasingPlanVOS1);
        }
        //采购计划审批
        if (permissions.contains("reagent_purchasing_plan_approve")) {

            LambdaQueryWrapper<PurchasingPlan> purchasingPlanLambdaQueryWrapper = new LambdaQueryWrapper<>();

            purchasingPlanLambdaQueryWrapper.eq(PurchasingPlan::getStatus, 4);

            List<PurchasingPlan> list = purchasingPlanService.list(purchasingPlanLambdaQueryWrapper);

            ArrayList<PurchasingPlanVO> purchasingPlanVOS1 = new ArrayList<>();

            for (PurchasingPlan purchasingPlan : list) {

                PurchasingPlanVO purchasingPlanVO = purchasingPlanService.getPurchasingPlanVO(purchasingPlan.getPurchasingPlanId());

                purchasingPlanVOS1.add(purchasingPlanVO);
            }
            purchasingPlanVOS.addAll(purchasingPlanVOS1);

        }
            //采购验收一级审核
        if (permissions.contains("reagent_acceptance_record_form_primary")){

            LambdaQueryWrapper<AcceptanceRecordForm> acceptanceRecordFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            acceptanceRecordFormLambdaQueryWrapper.eq(AcceptanceRecordForm::getStatus,1);

            List<AcceptanceRecordForm> list = acceptanceRecordFormService.list(acceptanceRecordFormLambdaQueryWrapper);

            ArrayList<AcceptanceRecordFormVO> acceptanceRecordFormVOS1 = new ArrayList<>();

            for (AcceptanceRecordForm acceptanceRecordForm : list) {

                AcceptanceRecordFormVO acceptanceRecordFormVO = acceptanceRecordFormService.getAcceptanceRecordFormVO(acceptanceRecordForm.getId());

                acceptanceRecordFormVOS1.add(acceptanceRecordFormVO);
            }
            acceptanceRecordFormVOS.addAll(acceptanceRecordFormVOS1);

        }

        //采购验收二级审核
        if (permissions.contains("reagent_acceptance_record_form_secondary")){

            LambdaQueryWrapper<AcceptanceRecordForm> acceptanceRecordFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            acceptanceRecordFormLambdaQueryWrapper.eq(AcceptanceRecordForm::getStatus,2);

            List<AcceptanceRecordForm> list = acceptanceRecordFormService.list(acceptanceRecordFormLambdaQueryWrapper);

            ArrayList<AcceptanceRecordFormVO> acceptanceRecordFormVOS1 = new ArrayList<>();

            for (AcceptanceRecordForm acceptanceRecordForm : list) {

                AcceptanceRecordFormVO acceptanceRecordFormVO = acceptanceRecordFormService.getAcceptanceRecordFormVO(acceptanceRecordForm.getId());

                acceptanceRecordFormVOS1.add(acceptanceRecordFormVO);
            }
            acceptanceRecordFormVOS.addAll(acceptanceRecordFormVOS1);

        }

        //采购验收三级审核
        if (permissions.contains("reagent_acceptance_record_form_threeLevel")){

            LambdaQueryWrapper<AcceptanceRecordForm> acceptanceRecordFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            acceptanceRecordFormLambdaQueryWrapper.eq(AcceptanceRecordForm::getStatus,3);

            List<AcceptanceRecordForm> list = acceptanceRecordFormService.list(acceptanceRecordFormLambdaQueryWrapper);

            ArrayList<AcceptanceRecordFormVO> acceptanceRecordFormVOS1 = new ArrayList<>();

            for (AcceptanceRecordForm acceptanceRecordForm : list) {

                AcceptanceRecordFormVO acceptanceRecordFormVO = acceptanceRecordFormService.getAcceptanceRecordFormVO(acceptanceRecordForm.getId());

                acceptanceRecordFormVOS1.add(acceptanceRecordFormVO);
            }
            acceptanceRecordFormVOS.addAll(acceptanceRecordFormVOS1);

        }
        //符合性检查一级审核
        if (permissions.contains("reagent_compliance_check_primaryAudit")){

            LambdaQueryWrapper<ComplianceCheck> complianceCheckLambdaQueryWrapper = new LambdaQueryWrapper<>();

            complianceCheckLambdaQueryWrapper.eq(ComplianceCheck::getStatus,1);

            List<ComplianceCheck> list = complianceCheckService.list(complianceCheckLambdaQueryWrapper);

            ArrayList<ComplianceCheckVO> complianceCheckVOS1 = new ArrayList<>();

            for (ComplianceCheck complianceCheck : list) {

                ComplianceCheckVO complianceCheckVO = complianceCheckService.getComplianceCheckVO(complianceCheck.getId());

                complianceCheckVOS1.add(complianceCheckVO);
            }
            complianceCheckVOS.addAll(complianceCheckVOS1);
        }

        //符合性检查二级审核
        if (permissions.contains("reagent_compliance_check_secondaryAudit")){

            LambdaQueryWrapper<ComplianceCheck> complianceCheckLambdaQueryWrapper = new LambdaQueryWrapper<>();

            complianceCheckLambdaQueryWrapper.eq(ComplianceCheck::getStatus,2);

            List<ComplianceCheck> list = complianceCheckService.list(complianceCheckLambdaQueryWrapper);

            ArrayList<ComplianceCheckVO> complianceCheckVOS1 = new ArrayList<>();

            for (ComplianceCheck complianceCheck : list) {

                ComplianceCheckVO complianceCheckVO = complianceCheckService.getComplianceCheckVO(complianceCheck.getId());

                complianceCheckVOS1.add(complianceCheckVO);
            }
            complianceCheckVOS.addAll(complianceCheckVOS1);
        }
        //期间核查计划审核
        if (permissions.contains("reagent_check_schedule_audit")){

            LambdaQueryWrapper<CheckSchedule> checkScheduleLambdaQueryWrapper = new LambdaQueryWrapper<>();

            checkScheduleLambdaQueryWrapper.eq(CheckSchedule::getStatus,1);

            List<CheckSchedule> list = checkScheduleService.list(checkScheduleLambdaQueryWrapper);

            ArrayList<CheckScheduleVO> checkScheduleVOS1 = new ArrayList<>();

            for (CheckSchedule checkSchedule : list) {

                CheckScheduleVO checkScheduleVO = checkScheduleService.getCheckScheduleVO(checkSchedule.getId());

                checkScheduleVOS1.add(checkScheduleVO);
            }
            checkScheduleVOS.addAll(checkScheduleVOS1);
        }
        //期间核查录入结果审核
        if (permissions.contains("reagent_period_verification_implementation_audit")){

            LambdaQueryWrapper<PeriodVerificationImplementation> periodVerificationImplementationLambdaQueryWrapper = new LambdaQueryWrapper<>();

            periodVerificationImplementationLambdaQueryWrapper.eq(PeriodVerificationImplementation::getCommitStatus,1);

            List<PeriodVerificationImplementation> list = periodVerificationImplementationService.list(periodVerificationImplementationLambdaQueryWrapper);

            ArrayList<PeriodVerificationImplementationVO> periodVerificationImplementationVOS1 = new ArrayList<>();

            for (PeriodVerificationImplementation periodVerificationImplementation : list) {

                PeriodVerificationImplementationVO periodVerificationImplementationVO = periodVerificationImplementationService.getPeriodVerificationImplementationVO(periodVerificationImplementation.getId());

                periodVerificationImplementationVOS1.add(periodVerificationImplementationVO);
            }
            periodVerificationImplementationVOS.addAll(periodVerificationImplementationVOS1);
        }

        //标准物质管理一级审核
        if (permissions.contains("reagent_standard_material_approval_form_auditPrimary")){

            LambdaQueryWrapper<StandardMaterialApprovalForm> standardMaterialApprovalFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            standardMaterialApprovalFormLambdaQueryWrapper.eq(StandardMaterialApprovalForm::getCommitStatus,1);

            List<StandardMaterialApprovalForm> list = standardMaterialApprovalFormService.list(standardMaterialApprovalFormLambdaQueryWrapper);

            ArrayList<StandardMaterialApprovalFormVO> standardMaterialApprovalFormVOS1 = new ArrayList<>();

            for (StandardMaterialApprovalForm standardMaterialApprovalForm : list) {

                StandardMaterialApprovalFormVO vo = standardMaterialApprovalFormService.getVO(standardMaterialApprovalForm.getId());

                standardMaterialApprovalFormVOS1.add(vo);
            }
            standardMaterialApprovalFormVOS.addAll(standardMaterialApprovalFormVOS1);
        }

        //标准物质管理二级审核
        if (permissions.contains("reagent_standard_material_approval_form_auditSecondary")){

            LambdaQueryWrapper<StandardMaterialApprovalForm> standardMaterialApprovalFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            standardMaterialApprovalFormLambdaQueryWrapper.eq(StandardMaterialApprovalForm::getCommitStatus,2);

            List<StandardMaterialApprovalForm> list = standardMaterialApprovalFormService.list(standardMaterialApprovalFormLambdaQueryWrapper);

            ArrayList<StandardMaterialApprovalFormVO> standardMaterialApprovalFormVOS1 = new ArrayList<>();

            for (StandardMaterialApprovalForm standardMaterialApprovalForm : list) {

                StandardMaterialApprovalFormVO vo = standardMaterialApprovalFormService.getVO(standardMaterialApprovalForm.getId());

                standardMaterialApprovalFormVOS1.add(vo);
            }
            standardMaterialApprovalFormVOS.addAll(standardMaterialApprovalFormVOS1);
        }
        //标准物质管理审批
        if (permissions.contains("reagent_standard_material_approval_form_approve")){

            LambdaQueryWrapper<StandardMaterialApprovalForm> standardMaterialApprovalFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            standardMaterialApprovalFormLambdaQueryWrapper.eq(StandardMaterialApprovalForm::getCommitStatus,3);

            List<StandardMaterialApprovalForm> list = standardMaterialApprovalFormService.list(standardMaterialApprovalFormLambdaQueryWrapper);

            ArrayList<StandardMaterialApprovalFormVO> standardMaterialApprovalFormVOS1 = new ArrayList<>();

            for (StandardMaterialApprovalForm standardMaterialApprovalForm : list) {

                StandardMaterialApprovalFormVO vo = standardMaterialApprovalFormService.getVO(standardMaterialApprovalForm.getId());

                standardMaterialApprovalFormVOS1.add(vo);
            }
            standardMaterialApprovalFormVOS.addAll(standardMaterialApprovalFormVOS1);
        }
        //供应商评价审核
        if (permissions.contains("reagent_evaluation_form_secondaryAudit")){

            LambdaQueryWrapper<EvaluationForm> evaluationFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            evaluationFormLambdaQueryWrapper.eq(EvaluationForm::getStatus,2);

            List<EvaluationForm> list = evaluationFormService.list(evaluationFormLambdaQueryWrapper);

            ArrayList<EvaluationFormVO> evaluationFormVOS1 = new ArrayList<>();

            for (EvaluationForm evaluationForm : list) {

                EvaluationFormVO formById = evaluationFormService.getFormById(evaluationForm.getId());

                evaluationFormVOS1.add(formById);
            }
            evaluationFormVOS.addAll(evaluationFormVOS1);
        }
        if (permissions.contains("reagent_evaluation_form_three_level_audit")){

            LambdaQueryWrapper<EvaluationForm> evaluationFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            evaluationFormLambdaQueryWrapper.eq(EvaluationForm::getStatus,3);

            List<EvaluationForm> list = evaluationFormService.list(evaluationFormLambdaQueryWrapper);

            ArrayList<EvaluationFormVO> evaluationFormVOS1 = new ArrayList<>();

            for (EvaluationForm evaluationForm : list) {

                EvaluationFormVO formById = evaluationFormService.getFormById(evaluationForm.getId());

                evaluationFormVOS1.add(formById);
            }
            evaluationFormVOS.addAll(evaluationFormVOS1);
        }
        //指导书审核列表
        if (permissions.contains("reagent_instruction_book_audit")){

            LambdaQueryWrapper<InstructionBook> instructionBookLambdaQueryWrapper = new LambdaQueryWrapper<>();

            instructionBookLambdaQueryWrapper.eq(InstructionBook::getCommitStatus,1);

            List<InstructionBook> list = instructionBookService.list(instructionBookLambdaQueryWrapper);

            ArrayList<InstructionBookVO> instructionBookVOS1 = new ArrayList<>();

            for (InstructionBook instructionBook : list) {

                InstructionBookVO vo = instructionBookService.getVO(instructionBook.getId());

                instructionBookVOS1.add(vo);
            }
            instructionBookVOS.addAll(instructionBookVOS1);

        }

        reviewAndApprove.setComplianceCheckVOList(complianceCheckVOS);
        reviewAndApprove.setDecentralizedRequestVOList(decentralizedRequestVOS);
        reviewAndApprove.setCheckScheduleVOList(checkScheduleVOS);
        reviewAndApprove.setEvaluationFormVOList(evaluationFormVOS);
        reviewAndApprove.setAcceptanceRecordFormVOList(acceptanceRecordFormVOS);
        reviewAndApprove.setPeriodVerificationImplementationVOList(periodVerificationImplementationVOS);
        reviewAndApprove.setPurchasingPlanVOList(purchasingPlanVOS);
        reviewAndApprove.setPurchaseCatalogueVOList(purchaseCatalogueVOList);
        reviewAndApprove.setStandardMaterialApprovalFormList(standardMaterialApprovalFormVOS);
        reviewAndApprove.setInstructionBookList(instructionBookVOS);

        return reviewAndApprove;
    }

    @Override
    public ReviewAndApprove getReviewAndApproveRecordList(DLPUser dlpUser) {

        Set<String> permissions = dlpUser.getPermissions();

        ReviewAndApprove reviewAndApprove = new ReviewAndApprove();
        //分散采购申请待审核列表
        List<DecentralizedRequestVO> decentralizedRequestVOS = new ArrayList<>();
        //采购目录待审核列表
        List<PurchaseCatalogueVO> purchaseCatalogueVOList = new ArrayList<>();
        //采购计划待审核列表
        List<PurchasingPlanVO> purchasingPlanVOS = new ArrayList<>();
        //符合性检查待审核列表
        List<ComplianceCheckVO> complianceCheckVOS = new ArrayList<>();
        //指导书待审核列表
        List<InstructionBookVO> instructionBookVOS = new ArrayList<>();
        //期间核查待审核列表
        List<CheckScheduleVO> checkScheduleVOS = new ArrayList<>();
        //标准物质管理待审核列表
        List<StandardMaterialApprovalFormVO> standardMaterialApprovalFormVOS = new ArrayList<>();
        //验收记录待审核列表
        List<AcceptanceRecordFormVO> acceptanceRecordFormVOS = new ArrayList<>();
        //期间核查结果录入待审核列表
        List<PeriodVerificationImplementationVO> periodVerificationImplementationVOS = new ArrayList<>();
        //供应商评价待审核列表
        List<EvaluationFormVO> evaluationFormVOS = new ArrayList<>();


        if (permissions.contains("reagent_decentralized_request_primary")) {

            //分散采购申请一级审核列表
            LambdaQueryWrapper<DecentralizedRequest> decentralizedRequestLambdaQueryWrapper = new LambdaQueryWrapper<>();

            decentralizedRequestLambdaQueryWrapper.eq(DecentralizedRequest::getStatus, 2).or().eq(DecentralizedRequest::getStatus, -2);

            List<DecentralizedRequest> decentralizedRequestList = decentralizedRequestService.list(decentralizedRequestLambdaQueryWrapper);

            List<DecentralizedRequestVO> decentralizedRequestVOList = new ArrayList<>();

            for (DecentralizedRequest decentralizedRequest : decentralizedRequestList) {

                DecentralizedRequestVO requestById = decentralizedRequestService.getRequestById(decentralizedRequest.getId());

                decentralizedRequestVOList.add(requestById);
            }
            decentralizedRequestVOS.addAll(decentralizedRequestVOList);
        }

        if (permissions.contains("reagent_decentralized_request_secondary")) {

            //分散采购申请二级审核列表
            LambdaQueryWrapper<DecentralizedRequest> decentralizedRequestLambdaQueryWrapper = new LambdaQueryWrapper<>();

            decentralizedRequestLambdaQueryWrapper.eq(DecentralizedRequest::getStatus, 3).or().eq(DecentralizedRequest::getStatus, -3);

            List<DecentralizedRequest> decentralizedRequestList = decentralizedRequestService.list(decentralizedRequestLambdaQueryWrapper);

            List<DecentralizedRequestVO> decentralizedRequestVOList = new ArrayList<>();

            for (DecentralizedRequest decentralizedRequest : decentralizedRequestList) {

                DecentralizedRequestVO requestById = decentralizedRequestService.getRequestById(decentralizedRequest.getId());

                decentralizedRequestVOList.add(requestById);
            }
            decentralizedRequestVOS.addAll(decentralizedRequestVOList);
        }

        if (permissions.contains("reagent_decentralized_request_threeLevel")) {

            //分散采购申请三级审核列表
            LambdaQueryWrapper<DecentralizedRequest> decentralizedRequestLambdaQueryWrapper = new LambdaQueryWrapper<>();

            decentralizedRequestLambdaQueryWrapper.eq(DecentralizedRequest::getStatus, 4).or().eq(DecentralizedRequest::getStatus, -4);

            List<DecentralizedRequest> decentralizedRequestList = decentralizedRequestService.list(decentralizedRequestLambdaQueryWrapper);

            List<DecentralizedRequestVO> decentralizedRequestVOList = new ArrayList<>();

            for (DecentralizedRequest decentralizedRequest : decentralizedRequestList) {

                DecentralizedRequestVO requestById = decentralizedRequestService.getRequestById(decentralizedRequest.getId());

                decentralizedRequestVOList.add(requestById);
            }
            decentralizedRequestVOS.addAll(decentralizedRequestVOList);
        }

        if (permissions.contains("reagent_decentralized_request_approve")) {
            //分散采购申请审批列表
            LambdaQueryWrapper<DecentralizedRequest> decentralizedRequestLambdaQueryWrapper = new LambdaQueryWrapper<>();

            decentralizedRequestLambdaQueryWrapper.eq(DecentralizedRequest::getStatus, 5).or().eq(DecentralizedRequest::getStatus, -5);

            List<DecentralizedRequest> decentralizedRequestList = decentralizedRequestService.list(decentralizedRequestLambdaQueryWrapper);

            List<DecentralizedRequestVO> decentralizedRequestVOList = new ArrayList<>();

            for (DecentralizedRequest decentralizedRequest : decentralizedRequestList) {

                DecentralizedRequestVO requestById = decentralizedRequestService.getRequestById(decentralizedRequest.getId());

                decentralizedRequestVOList.add(requestById);
            }
            decentralizedRequestVOS.addAll(decentralizedRequestVOList);
        }

        if (permissions.contains("reagent_purchase_catalogue_primary")) {
            //采购目录一级审核列表
            LambdaQueryWrapper<PurchaseCatalogue> purchaseCatalogueLambdaQueryWrapper = new LambdaQueryWrapper<>();

            purchaseCatalogueLambdaQueryWrapper.eq(PurchaseCatalogue::getStatus, 2).or().eq(PurchaseCatalogue::getStatus, -2);

            List<PurchaseCatalogue> list = purchaseCatalogueService.list(purchaseCatalogueLambdaQueryWrapper);

            ArrayList<PurchaseCatalogueVO> purchaseCatalogueVOArrayList = new ArrayList<>();

            for (PurchaseCatalogue purchaseCatalogue : list) {

                PurchaseCatalogueVO purchaseCatalogueVO = purchaseCatalogueService.getPurchaseCatalogueVO(purchaseCatalogue.getPurchaseCatalogueId());

                purchaseCatalogueVOArrayList.add(purchaseCatalogueVO);
            }
            purchaseCatalogueVOList.addAll(purchaseCatalogueVOArrayList);

        }
        //采购目录二级审核
        if (permissions.contains("reagent_purchase_catalogue_secondary")) {

            LambdaQueryWrapper<PurchaseCatalogue> purchaseCatalogueLambdaQueryWrapper = new LambdaQueryWrapper<>();

            purchaseCatalogueLambdaQueryWrapper.eq(PurchaseCatalogue::getStatus, 3).or().eq(PurchaseCatalogue::getStatus, -3);

            List<PurchaseCatalogue> list = purchaseCatalogueService.list(purchaseCatalogueLambdaQueryWrapper);

            ArrayList<PurchaseCatalogueVO> purchaseCatalogueVOArrayList = new ArrayList<>();

            for (PurchaseCatalogue purchaseCatalogue : list) {

                PurchaseCatalogueVO purchaseCatalogueVO = purchaseCatalogueService.getPurchaseCatalogueVO(purchaseCatalogue.getPurchaseCatalogueId());

                purchaseCatalogueVOArrayList.add(purchaseCatalogueVO);

            }
            purchaseCatalogueVOList.addAll(purchaseCatalogueVOArrayList);

        }
        //采购计划审核
        if (permissions.contains("reagent_purchasing_plan_audit")) {

            LambdaQueryWrapper<PurchasingPlan> purchasingPlanLambdaQueryWrapper = new LambdaQueryWrapper<>();

            purchasingPlanLambdaQueryWrapper.eq(PurchasingPlan::getStatus, 2).or().eq(PurchasingPlan::getStatus, -2);

            List<PurchasingPlan> list = purchasingPlanService.list(purchasingPlanLambdaQueryWrapper);

            ArrayList<PurchasingPlanVO> purchasingPlanVOS1 = new ArrayList<>();

            for (PurchasingPlan purchasingPlan : list) {

                PurchasingPlanVO purchasingPlanVO = purchasingPlanService.getPurchasingPlanVO(purchasingPlan.getPurchasingPlanId());

                purchasingPlanVOS1.add(purchasingPlanVO);
            }
            purchasingPlanVOS.addAll(purchasingPlanVOS1);
        }
        //采购计划审批
        if (permissions.contains("reagent_purchasing_plan_approve")) {

            LambdaQueryWrapper<PurchasingPlan> purchasingPlanLambdaQueryWrapper = new LambdaQueryWrapper<>();

            purchasingPlanLambdaQueryWrapper.eq(PurchasingPlan::getStatus, 5).or().eq(PurchasingPlan::getStatus, -5);

            List<PurchasingPlan> list = purchasingPlanService.list(purchasingPlanLambdaQueryWrapper);

            ArrayList<PurchasingPlanVO> purchasingPlanVOS1 = new ArrayList<>();

            for (PurchasingPlan purchasingPlan : list) {

                PurchasingPlanVO purchasingPlanVO = purchasingPlanService.getPurchasingPlanVO(purchasingPlan.getPurchasingPlanId());

                purchasingPlanVOS1.add(purchasingPlanVO);
            }
            purchasingPlanVOS.addAll(purchasingPlanVOS1);

        }
        //采购验收一级审核
        if (permissions.contains("reagent_acceptance_record_form_primary")){

            LambdaQueryWrapper<AcceptanceRecordForm> acceptanceRecordFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            acceptanceRecordFormLambdaQueryWrapper.eq(AcceptanceRecordForm::getStatus,2).or().eq(AcceptanceRecordForm::getStatus, -2);

            List<AcceptanceRecordForm> list = acceptanceRecordFormService.list(acceptanceRecordFormLambdaQueryWrapper);

            ArrayList<AcceptanceRecordFormVO> acceptanceRecordFormVOS1 = new ArrayList<>();

            for (AcceptanceRecordForm acceptanceRecordForm : list) {

                AcceptanceRecordFormVO acceptanceRecordFormVO = acceptanceRecordFormService.getAcceptanceRecordFormVO(acceptanceRecordForm.getId());

                acceptanceRecordFormVOS1.add(acceptanceRecordFormVO);
            }
            acceptanceRecordFormVOS.addAll(acceptanceRecordFormVOS1);

        }

        //采购验收二级审核
        if (permissions.contains("reagent_acceptance_record_form_secondary")){

            LambdaQueryWrapper<AcceptanceRecordForm> acceptanceRecordFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            acceptanceRecordFormLambdaQueryWrapper.eq(AcceptanceRecordForm::getStatus,3).or().eq(AcceptanceRecordForm::getStatus, -3);

            List<AcceptanceRecordForm> list = acceptanceRecordFormService.list(acceptanceRecordFormLambdaQueryWrapper);

            ArrayList<AcceptanceRecordFormVO> acceptanceRecordFormVOS1 = new ArrayList<>();

            for (AcceptanceRecordForm acceptanceRecordForm : list) {

                AcceptanceRecordFormVO acceptanceRecordFormVO = acceptanceRecordFormService.getAcceptanceRecordFormVO(acceptanceRecordForm.getId());

                acceptanceRecordFormVOS1.add(acceptanceRecordFormVO);
            }
            acceptanceRecordFormVOS.addAll(acceptanceRecordFormVOS1);

        }

        //采购验收三级审核
        if (permissions.contains("reagent_acceptance_record_form_threeLevel")){

            LambdaQueryWrapper<AcceptanceRecordForm> acceptanceRecordFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            acceptanceRecordFormLambdaQueryWrapper.eq(AcceptanceRecordForm::getStatus,4).or().eq(AcceptanceRecordForm::getStatus, -4);

            List<AcceptanceRecordForm> list = acceptanceRecordFormService.list(acceptanceRecordFormLambdaQueryWrapper);

            ArrayList<AcceptanceRecordFormVO> acceptanceRecordFormVOS1 = new ArrayList<>();

            for (AcceptanceRecordForm acceptanceRecordForm : list) {

                AcceptanceRecordFormVO acceptanceRecordFormVO = acceptanceRecordFormService.getAcceptanceRecordFormVO(acceptanceRecordForm.getId());

                acceptanceRecordFormVOS1.add(acceptanceRecordFormVO);
            }
            acceptanceRecordFormVOS.addAll(acceptanceRecordFormVOS1);

        }
        //符合性检查一级审核
        if (permissions.contains("reagent_compliance_check_primaryAudit")){

            LambdaQueryWrapper<ComplianceCheck> complianceCheckLambdaQueryWrapper = new LambdaQueryWrapper<>();

            complianceCheckLambdaQueryWrapper.eq(ComplianceCheck::getStatus,2).or().eq(ComplianceCheck::getStatus, -2);

            List<ComplianceCheck> list = complianceCheckService.list(complianceCheckLambdaQueryWrapper);

            ArrayList<ComplianceCheckVO> complianceCheckVOS1 = new ArrayList<>();

            for (ComplianceCheck complianceCheck : list) {

                ComplianceCheckVO complianceCheckVO = complianceCheckService.getComplianceCheckVO(complianceCheck.getId());

                complianceCheckVOS1.add(complianceCheckVO);
            }
            complianceCheckVOS.addAll(complianceCheckVOS1);
        }

        //符合性检查二级审核
        if (permissions.contains("reagent_compliance_check_secondaryAudit")){

            LambdaQueryWrapper<ComplianceCheck> complianceCheckLambdaQueryWrapper = new LambdaQueryWrapper<>();

            complianceCheckLambdaQueryWrapper.eq(ComplianceCheck::getStatus,3).or().eq(ComplianceCheck::getStatus, -3);

            List<ComplianceCheck> list = complianceCheckService.list(complianceCheckLambdaQueryWrapper);

            ArrayList<ComplianceCheckVO> complianceCheckVOS1 = new ArrayList<>();

            for (ComplianceCheck complianceCheck : list) {

                ComplianceCheckVO complianceCheckVO = complianceCheckService.getComplianceCheckVO(complianceCheck.getId());

                complianceCheckVOS1.add(complianceCheckVO);
            }
            complianceCheckVOS.addAll(complianceCheckVOS1);
        }
        //期间核查计划审核
        if (permissions.contains("reagent_check_schedule_audit")){

            LambdaQueryWrapper<CheckSchedule> checkScheduleLambdaQueryWrapper = new LambdaQueryWrapper<>();

            checkScheduleLambdaQueryWrapper.eq(CheckSchedule::getStatus,2).or().eq(CheckSchedule::getStatus, -2);

            List<CheckSchedule> list = checkScheduleService.list(checkScheduleLambdaQueryWrapper);

            ArrayList<CheckScheduleVO> checkScheduleVOS1 = new ArrayList<>();

            for (CheckSchedule checkSchedule : list) {

                CheckScheduleVO checkScheduleVO = checkScheduleService.getCheckScheduleVO(checkSchedule.getId());

                checkScheduleVOS1.add(checkScheduleVO);
            }
            checkScheduleVOS.addAll(checkScheduleVOS1);
        }
        //期间核查录入结果审核
        if (permissions.contains("reagent_period_verification_implementation_audit")){

            LambdaQueryWrapper<PeriodVerificationImplementation> periodVerificationImplementationLambdaQueryWrapper = new LambdaQueryWrapper<>();

            periodVerificationImplementationLambdaQueryWrapper.eq(PeriodVerificationImplementation::getCommitStatus,2).or().eq(PeriodVerificationImplementation::getCommitStatus, -2);

            List<PeriodVerificationImplementation> list = periodVerificationImplementationService.list(periodVerificationImplementationLambdaQueryWrapper);

            ArrayList<PeriodVerificationImplementationVO> periodVerificationImplementationVOS1 = new ArrayList<>();

            for (PeriodVerificationImplementation periodVerificationImplementation : list) {

                PeriodVerificationImplementationVO periodVerificationImplementationVO = periodVerificationImplementationService.getPeriodVerificationImplementationVO(periodVerificationImplementation.getId());

                periodVerificationImplementationVOS1.add(periodVerificationImplementationVO);
            }
            periodVerificationImplementationVOS.addAll(periodVerificationImplementationVOS1);
        }

        //标准物质管理一级审核
        if (permissions.contains("reagent_standard_material_approval_form_auditPrimary")){

            LambdaQueryWrapper<StandardMaterialApprovalForm> standardMaterialApprovalFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            standardMaterialApprovalFormLambdaQueryWrapper.eq(StandardMaterialApprovalForm::getCommitStatus,2).or().eq(StandardMaterialApprovalForm::getCommitStatus, -2);

            List<StandardMaterialApprovalForm> list = standardMaterialApprovalFormService.list(standardMaterialApprovalFormLambdaQueryWrapper);

            ArrayList<StandardMaterialApprovalFormVO> standardMaterialApprovalFormVOS1 = new ArrayList<>();

            for (StandardMaterialApprovalForm standardMaterialApprovalForm : list) {

                StandardMaterialApprovalFormVO vo = standardMaterialApprovalFormService.getVO(standardMaterialApprovalForm.getId());

                standardMaterialApprovalFormVOS1.add(vo);
            }
            standardMaterialApprovalFormVOS.addAll(standardMaterialApprovalFormVOS1);
        }

        //标准物质管理二级审核
        if (permissions.contains("reagent_standard_material_approval_form_auditSecondary")){

            LambdaQueryWrapper<StandardMaterialApprovalForm> standardMaterialApprovalFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            standardMaterialApprovalFormLambdaQueryWrapper.eq(StandardMaterialApprovalForm::getCommitStatus,3).or().eq(StandardMaterialApprovalForm::getCommitStatus, -3);

            List<StandardMaterialApprovalForm> list = standardMaterialApprovalFormService.list(standardMaterialApprovalFormLambdaQueryWrapper);

            ArrayList<StandardMaterialApprovalFormVO> standardMaterialApprovalFormVOS1 = new ArrayList<>();

            for (StandardMaterialApprovalForm standardMaterialApprovalForm : list) {

                StandardMaterialApprovalFormVO vo = standardMaterialApprovalFormService.getVO(standardMaterialApprovalForm.getId());

                standardMaterialApprovalFormVOS1.add(vo);
            }
            standardMaterialApprovalFormVOS.addAll(standardMaterialApprovalFormVOS1);
        }
        //标准物质管理审批
        if (permissions.contains("reagent_standard_material_approval_form_approve")){

            LambdaQueryWrapper<StandardMaterialApprovalForm> standardMaterialApprovalFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            standardMaterialApprovalFormLambdaQueryWrapper.eq(StandardMaterialApprovalForm::getCommitStatus,4).or().eq(StandardMaterialApprovalForm::getCommitStatus, -4);

            List<StandardMaterialApprovalForm> list = standardMaterialApprovalFormService.list(standardMaterialApprovalFormLambdaQueryWrapper);

            ArrayList<StandardMaterialApprovalFormVO> standardMaterialApprovalFormVOS1 = new ArrayList<>();

            for (StandardMaterialApprovalForm standardMaterialApprovalForm : list) {

                StandardMaterialApprovalFormVO vo = standardMaterialApprovalFormService.getVO(standardMaterialApprovalForm.getId());

                standardMaterialApprovalFormVOS1.add(vo);
            }
            standardMaterialApprovalFormVOS.addAll(standardMaterialApprovalFormVOS1);
        }

        if (permissions.contains("reagent_evaluation_form_secondaryAudit")){

            LambdaQueryWrapper<EvaluationForm> evaluationFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            evaluationFormLambdaQueryWrapper.eq(EvaluationForm::getStatus,3).or().eq(EvaluationForm::getStatus, -3);

            List<EvaluationForm> list = evaluationFormService.list(evaluationFormLambdaQueryWrapper);

            ArrayList<EvaluationFormVO> evaluationFormVOS1 = new ArrayList<>();

            for (EvaluationForm evaluationForm : list) {

                EvaluationFormVO formById = evaluationFormService.getFormById(evaluationForm.getId());

                evaluationFormVOS1.add(formById);
            }
            evaluationFormVOS.addAll(evaluationFormVOS1);
        }

        if (permissions.contains("reagent_evaluation_form_three_level_audit")){

            LambdaQueryWrapper<EvaluationForm> evaluationFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            evaluationFormLambdaQueryWrapper.eq(EvaluationForm::getStatus,4).or().eq(EvaluationForm::getStatus, -4);

            List<EvaluationForm> list = evaluationFormService.list(evaluationFormLambdaQueryWrapper);

            ArrayList<EvaluationFormVO> evaluationFormVOS1 = new ArrayList<>();

            for (EvaluationForm evaluationForm : list) {

                EvaluationFormVO formById = evaluationFormService.getFormById(evaluationForm.getId());

                evaluationFormVOS1.add(formById);
            }
            evaluationFormVOS.addAll(evaluationFormVOS1);
        }
        //指导书审核列表
        if (permissions.contains("reagent_instruction_book_audit")){

            LambdaQueryWrapper<InstructionBook> instructionBookLambdaQueryWrapper = new LambdaQueryWrapper<>();

            instructionBookLambdaQueryWrapper.eq(InstructionBook::getCommitStatus,2).or().eq(InstructionBook::getCommitStatus, -2);

            List<InstructionBook> list = instructionBookService.list(instructionBookLambdaQueryWrapper);

            ArrayList<InstructionBookVO> instructionBookVOS1 = new ArrayList<>();

            for (InstructionBook instructionBook : list) {

                InstructionBookVO vo = instructionBookService.getVO(instructionBook.getId());

                instructionBookVOS1.add(vo);
            }
            instructionBookVOS.addAll(instructionBookVOS1);

        }

        reviewAndApprove.setComplianceCheckVOList(complianceCheckVOS);
        reviewAndApprove.setDecentralizedRequestVOList(decentralizedRequestVOS);
        reviewAndApprove.setCheckScheduleVOList(checkScheduleVOS);
        reviewAndApprove.setEvaluationFormVOList(evaluationFormVOS);
        reviewAndApprove.setAcceptanceRecordFormVOList(acceptanceRecordFormVOS);
        reviewAndApprove.setPeriodVerificationImplementationVOList(periodVerificationImplementationVOS);
        reviewAndApprove.setPurchasingPlanVOList(purchasingPlanVOS);
        reviewAndApprove.setPurchaseCatalogueVOList(purchaseCatalogueVOList);
        reviewAndApprove.setStandardMaterialApprovalFormList(standardMaterialApprovalFormVOS);
        reviewAndApprove.setInstructionBookList(instructionBookVOS);

        return reviewAndApprove;
    }

}
