package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.reagent.dto.StandardMaterialApplicationDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.StandardMaterialApplicationMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.StandardMaterialApplicationVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * (标准物质领用/归还登记表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质领用 / 归还登记表) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class StandardMaterialApplicationServiceImpl extends ServiceImpl<StandardMaterialApplicationMapper, StandardMaterialApplication> implements StandardMaterialApplicationService {

    @Autowired
    private StandardMaterialApplicationService standardMaterialApplicationService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;
    @Autowired
    private SolutionUseFormService solutionUseFormService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;


    @Autowired
    private StandardReserveSolutionService standardReserveSolutionService;

    @Override//归还标准物质
    @Transactional
    public StandardMaterialApplication returnById(StandardMaterialApplicationDTO standardMaterialApplicationDTO) {

        StandardMaterialApplication byId = standardMaterialApplicationService.getById(standardMaterialApplicationDTO.getStandardMaterialApplicationId());

        byId.setPurposeAndQuantity(standardMaterialApplicationDTO.getPurposeAndQuantity());
        byId.setDateOfReturn(LocalDateTime.now());
        byId.setStatus(1);
        //若无需归还，则证明标准物质/标准储备溶液已经使用完毕，删除记录即可
        if (!standardMaterialApplicationDTO.getOutOfUse()){

            ReferenceMaterial referenceMaterial = referenceMaterialService.getById(byId.getReferenceMaterialId());

            referenceMaterial.setStatus(-4);
            referenceMaterialService.updateById(referenceMaterial);
        }

        LambdaQueryWrapper<SolutionUseForm> solutionUseFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

        solutionUseFormLambdaQueryWrapper.eq(SolutionUseForm::getStandardMaterialApplicationId, byId.getStandardMaterialApplicationId());

        SolutionUseForm one = solutionUseFormService.getOne(solutionUseFormLambdaQueryWrapper);
        //如果是标准储备溶液，则需补充完善标准储备溶液使用记录表的使用数量
        if (one != null) {
            one.setQuantityUsed(byId.getPurposeAndQuantity());
            solutionUseFormService.updateById(one);
        }
//
        if (standardMaterialApplicationService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("归还失败"));
    }

    @Override//领用记录
    public IPage<StandardMaterialApplicationVO> getPage(IPage<StandardMaterialApplication> page, QueryWrapper<StandardMaterialApplication> qw) {

        IPage<StandardMaterialApplicationVO> standardMaterialApplicationVOPage = baseMapper.getStandardMaterialApplicationVOPage(page, qw);

        return standardMaterialApplicationVOPage;

    }

    //归还记录
    @Override//归还记录
    public IPage<StandardMaterialApplicationVO> getList(IPage<StandardMaterialApplication> page, QueryWrapper<StandardMaterialApplication> qw) {

        IPage<StandardMaterialApplicationVO> standardMaterialApplicationVOList = baseMapper.getStandardMaterialApplicationVOList(page, qw);

        List<StandardMaterialApplicationVO> records = standardMaterialApplicationVOList.getRecords();

        for (StandardMaterialApplicationVO record : records) {

            ReagentConsumables byId = reagentConsumablesService.getById(record.getReferenceSubstanceId());

            record.setCategory(byId.getCategory());

            LambdaQueryWrapper<StandardReserveSolution> standardReserveSolutionLambdaQueryWrapper = new LambdaQueryWrapper<>();

            standardReserveSolutionLambdaQueryWrapper.eq(StandardReserveSolution::getSolutionNumbering,record.getReferenceMaterialNumber());

            StandardReserveSolution one = standardReserveSolutionService.getOne(standardReserveSolutionLambdaQueryWrapper);

            record.setReagentConsumables(byId);

            if (one!=null){
                String configurationConcentration = one.getConfigurationConcentration();
                record.setConfigurationConcentration(configurationConcentration);

            }



        }
        return standardMaterialApplicationVOList;
    }
}

