package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.util.PageUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.config.PageUtils;
import digital.laboratory.platform.reagent.dto.WarehousingRecordFormDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.WarehousingRecordFormMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.WarehousingContentVO;
import digital.laboratory.platform.reagent.vo.WarehousingRecordFormVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

/**
 * 签收记录表服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 签收记录表 服务实现类
 */
@Service
@SuppressWarnings("all")
public class WarehousingRecordFormServiceImpl extends ServiceImpl<WarehousingRecordFormMapper, WarehousingRecordForm> implements WarehousingRecordFormService {
    @Autowired
    private WarehousingRecordFormService warehousingRecordFormService;

    @Autowired
    private WarehousingContentService warehousingContentService;

    @Autowired
    private WarehousingBatchListService warehousingBatchListService;

    @Autowired
    private StorageRegistrationFormService storageRegistrationFormService;

    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;
    @Autowired
    private ReferenceMaterialService referenceMaterialService;
    @Autowired
    private LatticeFormService latticeFormService;

    @Autowired
    private StorageRoomFormService storageRoomFormService;

    @Autowired
    private CabinetFormService cabinetFormService;

    @Autowired
    private BatchDetailsService batchDetailsService;

    @Autowired
    private AcceptanceRecordFormService acceptanceRecordFormService;
    @Autowired
    private ProvideServicesOrSuppliesService provideServicesOrSuppliesService;

    @Autowired
    private EvaluationFormService evaluationFormService;

    @Autowired
    private SupplierInformationService supplierInformationService;

    @Override//查看采购入库
    public WarehousingRecordFormVO getWarehousingRecordFormVO(String warehousingFormId) {

        WarehousingRecordForm byId = warehousingRecordFormService.getById(warehousingFormId);

        WarehousingRecordFormVO warehousingRecordFormVO = new WarehousingRecordFormVO();

        BeanUtils.copyProperties(byId, warehousingRecordFormVO);

        List<WarehousingContentVO> warehousingContentVOList = warehousingContentService.getWarehousingContentVOList(warehousingFormId);

        warehousingRecordFormVO.setWarehousingContentVOList(warehousingContentVOList);

        return warehousingRecordFormVO;
    }


    @Transactional
    @Override//录入入库明细
    public WarehousingRecordFormVO addFormById(List<WarehousingRecordFormDTO> warehousingRecordFormDTOList, DLPUser dlpUser) {

        boolean flag = true;

        int i = 0;
        //签收内容循环
        for (WarehousingRecordFormDTO warehousingRecordFormDTO : warehousingRecordFormDTOList) {
            //录入签收批次信息
            WarehousingBatchList warehousingBatchList = new WarehousingBatchList();

            BeanUtils.copyProperties(warehousingRecordFormDTO, warehousingBatchList);

            warehousingBatchList.setId(IdWorker.get32UUID().toUpperCase());

            warehousingBatchList.setDepositorId(dlpUser.getId());

            warehousingBatchList.setDateOfReceipt(LocalDateTime.now());

            LambdaQueryWrapper<WarehousingBatchList> warehousingBatchListLambdaQueryWrapper = new LambdaQueryWrapper<>();

            warehousingBatchListLambdaQueryWrapper.eq(WarehousingBatchList::getWarehousingContentId, warehousingBatchList.getWarehousingContentId());
            //通过判断签收内容表里的签收批次表数量，进行批次字段的赋值
            warehousingBatchList.setBatch(warehousingBatchListService.list(warehousingBatchListLambdaQueryWrapper).size() + 1);

            WarehousingContent byId = warehousingContentService.getById(warehousingBatchList.getWarehousingContentId());


            byId.setWarehousingQuantity(byId.getWarehousingQuantity() + warehousingBatchList.getQuantity());
            //判断采购数量与签收数量
            if (byId.getWarehousingQuantity() > byId.getTotalQuantity()) {
                throw new RuntimeException(String.format("入库数量不能大于采购数量"));
            }

            if (warehousingContentService.updateById(byId) & warehousingBatchListService.save(warehousingBatchList)) {

                flag = true;
            } else {
                flag = false;
            }
            //入库登记表更新
            StorageRegistrationForm storageRegistrationForm = new StorageRegistrationForm();

            storageRegistrationForm.setStorageRegistrationFormId(IdWorker.get32UUID().toUpperCase());

            storageRegistrationForm.setReagentConsumableId(byId.getReagentConsumableId());

            storageRegistrationForm.setQuantity(warehousingBatchList.getQuantity());

            storageRegistrationForm.setRemarks(warehousingRecordFormDTO.getRemark());

            storageRegistrationFormService.save(storageRegistrationForm);

            LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

            reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, byId.getReagentConsumableId());

            ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);
            //录入仓库批次信息，若仓库不存在该物品，则新增该物品的库存信息
            if (one == null) {

                ReagentConsumableInventory reagentConsumableInventory = new ReagentConsumableInventory();

                LambdaQueryWrapper<ReagentConsumables> reagentConsumablesLambdaQueryWrapper = new LambdaQueryWrapper<>();

                reagentConsumablesLambdaQueryWrapper.eq(ReagentConsumables::getReagentConsumableId, byId.getReagentConsumableId());

                ReagentConsumables reagentConsumables = reagentConsumablesService.getOne(reagentConsumablesLambdaQueryWrapper);

                BeanUtils.copyProperties(reagentConsumables, reagentConsumableInventory);

                reagentConsumableInventory.setReagentConsumableInventoryId(IdWorker.get32UUID().toUpperCase());
                reagentConsumableInventory.setStatus(1);
                reagentConsumableInventory.setCreateTime(LocalDateTime.now());
                reagentConsumableInventory.setUpdateTime(LocalDateTime.now());
                reagentConsumableInventory.setWarningValue(warehousingRecordFormDTO.getWarning_value());

                BatchDetails batchDetails = new BatchDetails();

                BeanUtils.copyProperties(warehousingRecordFormDTO, batchDetails);

                batchDetails.setBatchDetailsId(IdWorker.get32UUID().toUpperCase());

                batchDetails.setServiceStatus(1);

                batchDetails.setPurchaseTime(LocalDateTime.now());

                batchDetails.setReagentConsumableInventoryId(reagentConsumableInventory.getReagentConsumableInventoryId());

                batchDetails.setSupplierId(byId.getSupplierId());

                if (reagentConsumableInventory.getTotalQuantity() == null) {

                    reagentConsumableInventory.setTotalQuantity(0);
                }

                if (reagentConsumableInventory.getCategory().equals("试剂") & reagentConsumableInventory.getCategory().equals("标准物质")) {

                    reagentConsumableInventory.setTotalQuantity(reagentConsumableInventory.getTotalQuantity() + batchDetails.getQuantity() * Integer.valueOf(reagentConsumableInventory.getPackagedCopies()));

                } else {

                    reagentConsumableInventory.setTotalQuantity(reagentConsumableInventory.getTotalQuantity() + batchDetails.getQuantity());
                }


                reagentConsumableInventory.setTotalQuantity(reagentConsumableInventory.getTotalQuantity() + batchDetails.getQuantity());
                //如果是标准物质，则赋值编号
                if (reagentConsumables.getCategory().equals("标准物质")) {

                    for (int j = 0; j < batchDetails.getQuantity(); j++) {

                        ReferenceMaterial referenceMaterial = new ReferenceMaterial();

                        referenceMaterial.setId(IdWorker.get32UUID().toUpperCase());

                        referenceMaterial.setReagentConsumableId(reagentConsumables.getReagentConsumableId());

                        referenceMaterial.setBatchDetailsId(batchDetails.getBatchDetailsId());

                        referenceMaterial.setReagentConsumableInventoryId(reagentConsumableInventory.getReagentConsumableInventoryId());

                        referenceMaterial.setStatus(0);

                        Calendar calendar = Calendar.getInstance();

                        int year = calendar.get(Calendar.YEAR);

                        LatticeForm latticeForm = latticeFormService.getById(warehousingRecordFormDTO.getLatticeId());

                        CabinetForm cabinetForm = cabinetFormService.getById(latticeForm.getCabinetFormId());

                        StorageRoomForm storageRoomForm = storageRoomFormService.getById(cabinetForm.getStorageRoomFormId());

                        String prefix = reagentConsumables.getEnglishName() + "-" + year + "-" + storageRoomForm.getFloor() + "-";

                        List<ReferenceMaterial> list = referenceMaterialService.list(Wrappers.<ReferenceMaterial>query()
                                .likeRight("number", prefix)
                                .orderByDesc("number"));

                        int newNo = 1;

                        if ((list != null) && (list.size() > 0)) {
                            ReferenceMaterial referenceMaterial1 = list.get(0);
                            String strMaxNo = StrUtil.removePrefixIgnoreCase(referenceMaterial1.getNumber(), prefix);
                            try {
                                int maxno = Integer.parseUnsignedInt(strMaxNo);
                                newNo = maxno + 1;
                            } catch (NumberFormatException e) {
                                // 如果后缀有非数字, 则无视之, 重头编码
                                newNo = 1;
                            }
                        }
                        referenceMaterial.setNumber(prefix + String.format("%03d", newNo));

                        referenceMaterialService.save(referenceMaterial);

                    }

                }
                reagentConsumableInventoryService.save(reagentConsumableInventory);

                batchDetailsService.save(batchDetails);
            } else {
                //如果仓库存在该物品

                LambdaQueryWrapper<ReagentConsumables> reagentConsumablesLambdaQueryWrapper = new LambdaQueryWrapper<>();

                reagentConsumablesLambdaQueryWrapper.eq(ReagentConsumables::getReagentConsumableId, byId.getReagentConsumableId());

                ReagentConsumables reagentConsumables = reagentConsumablesService.getOne(reagentConsumablesLambdaQueryWrapper);

                BatchDetails batchDetails = new BatchDetails();

                BeanUtils.copyProperties(warehousingRecordFormDTO, batchDetails);

                batchDetails.setBatchDetailsId(IdWorker.get32UUID().toUpperCase());

                batchDetails.setServiceStatus(1);

                batchDetails.setPurchaseTime(LocalDateTime.now());

                batchDetails.setSupplierId(byId.getSupplierId());

                batchDetails.setReagentConsumableInventoryId(one.getReagentConsumableInventoryId());

                one.setTotalQuantity(one.getTotalQuantity() + batchDetails.getQuantity());
                one.setWarningValue(warehousingRecordFormDTO.getWarning_value());

                if (one.getCategory().equals("标准物质")) {

                    for (int j = 0; j < batchDetails.getQuantity(); j++) {

                        ReferenceMaterial referenceMaterial = new ReferenceMaterial();

                        referenceMaterial.setId(IdWorker.get32UUID().toUpperCase());

                        referenceMaterial.setReagentConsumableId(one.getReagentConsumableId());

                        referenceMaterial.setBatchDetailsId(batchDetails.getBatchDetailsId());

                        referenceMaterial.setReagentConsumableInventoryId(one.getReagentConsumableInventoryId());

                        referenceMaterial.setStatus(0);

                        Calendar calendar = Calendar.getInstance();

                        int year = calendar.get(Calendar.YEAR);

                        LatticeForm latticeForm = latticeFormService.getById(warehousingRecordFormDTO.getLatticeId());

                        CabinetForm cabinetForm = cabinetFormService.getById(latticeForm.getCabinetFormId());

                        StorageRoomForm storageRoomForm = storageRoomFormService.getById(cabinetForm.getStorageRoomFormId());

                        String prefix = reagentConsumables.getEnglishName() + "-" + year + "-" + storageRoomForm.getFloor() + "-";

                        List<ReferenceMaterial> list = referenceMaterialService.list(Wrappers.<ReferenceMaterial>query()
                                .likeRight("number", prefix)
                                .orderByDesc("number"));

                        int newNo = 1;

                        if ((list != null) && (list.size() > 0)) {
                            ReferenceMaterial referenceMaterial1 = list.get(0);
                            String strMaxNo = StrUtil.removePrefixIgnoreCase(referenceMaterial1.getNumber(), prefix);
                            try {
                                int maxno = Integer.parseUnsignedInt(strMaxNo);
                                newNo = maxno + 1;
                            } catch (NumberFormatException e) {
                                // 如果后缀有非数字, 则无视之, 重头编码
                                newNo = 1;
                            }
                        }
                        referenceMaterial.setNumber(prefix + String.format("%03d", newNo));

                        referenceMaterialService.save(referenceMaterial);
                    }

                }
                reagentConsumableInventoryService.updateById(one);

                batchDetailsService.save(batchDetails);
            }
        }
        LambdaQueryWrapper<WarehousingContent> warehousingContentLambdaQueryWrapper = new LambdaQueryWrapper<>();

        WarehousingContent byId = warehousingContentService.getById(warehousingRecordFormDTOList.get(0).getWarehousingContentId());

        String warehousingRecordFormId = byId.getWarehousingRecordFormId();

        warehousingContentLambdaQueryWrapper.eq(WarehousingContent::getWarehousingRecordFormId, warehousingRecordFormId);

        List<WarehousingContent> list = warehousingContentService.list(warehousingContentLambdaQueryWrapper);

        WarehousingRecordForm warehousingRecordForm = warehousingRecordFormService.getById(warehousingRecordFormId);
//遍历采购内容，判断试剂耗材是否入库完毕
        for (WarehousingContent warehousingContent : list) {

            if (warehousingContent.getWarehousingQuantity() == warehousingContent.getTotalQuantity()) {

                LambdaQueryWrapper<WarehousingBatchList> warehousingBatchListLambdaQueryWrapper = new LambdaQueryWrapper<>();

                warehousingBatchListLambdaQueryWrapper.eq(WarehousingBatchList::getWarehousingContentId, warehousingContent.getId());

                List<WarehousingBatchList> list1 = warehousingBatchListService.list(warehousingBatchListLambdaQueryWrapper);

                if (warehousingContent.getAcceptanceRecordFormId() == null) {

                    AcceptanceRecordForm acceptanceRecordForm = acceptanceRecordFormService.addForm(warehousingContent.getReagentConsumableId(), warehousingContent.getSupplierId());

                    warehousingContent.setAcceptanceRecordFormId(acceptanceRecordForm.getId());

                    warehousingContentService.updateById(warehousingContent);
                }

                i = i + 1;
            }
        }
        //判断该清单所有试剂耗材是否入库完成
        if (i == list.size()) {

            List<SupplierInformation> supplierInformations = new ArrayList<>();

            String evaluationFormId = null;
            //创建供应商评价表
            for (WarehousingContent warehousingContent : list) {

                SupplierInformation supplierInformation = supplierInformationService.getById(warehousingContent.getSupplierId());

                if (supplierInformations.size()==0){
                    supplierInformations.add(supplierInformation);
                    EvaluationForm evaluationForm = evaluationFormService.addForm(supplierInformation.getId());
                    provideServicesOrSuppliesService.addById(evaluationForm.getId(),warehousingContent.getReagentConsumableId());
                    evaluationFormId=evaluationForm.getId();
                    warehousingContent.setEvaluationFormId(evaluationForm.getId());

                }else if (!(supplierInformations.contains(supplierInformation))){
                    supplierInformations.add(supplierInformation);
                    EvaluationForm evaluationForm = evaluationFormService.addForm(supplierInformation.getId());
                    warehousingContent.setEvaluationFormId(evaluationForm.getId());
                    evaluationFormId=evaluationForm.getId();
                    provideServicesOrSuppliesService.addById(evaluationForm.getId(),warehousingContent.getReagentConsumableId());
                }else {
                    provideServicesOrSuppliesService.addById(evaluationFormId,warehousingContent.getReagentConsumableId());
                }
            }

            for (SupplierInformation supplierInformation : supplierInformations) {

                evaluationFormService.addForm(supplierInformation.getId());



            }

            warehousingRecordForm.setStatus(2);


        } else {
            warehousingRecordForm.setStatus(1);
        }

        warehousingRecordFormService.updateById(warehousingRecordForm);

        WarehousingRecordFormVO warehousingRecordFormVO = warehousingRecordFormService.getWarehousingRecordFormVO(warehousingRecordFormId);

        return warehousingRecordFormVO;
    }

    @Override
    public Page<WarehousingRecordFormVO> getWarehousingRecordFormVOList(Page page) {

        List<WarehousingRecordFormVO> warehousingRecordFormVOList = baseMapper.getWarehousingRecordFormVOList();

        for (WarehousingRecordFormVO warehousingRecordFormVO : warehousingRecordFormVOList) {

            LambdaQueryWrapper<WarehousingContent> warehousingContentLambdaQueryWrapper = new LambdaQueryWrapper<>();

            warehousingContentLambdaQueryWrapper.eq(WarehousingContent::getWarehousingRecordFormId, warehousingRecordFormVO.getId());

            List<WarehousingContent> list = warehousingContentService.list(warehousingContentLambdaQueryWrapper);

            Integer x = 0;
            Integer y = 0;
            for (WarehousingContent warehousingContent : list) {
                Integer totalQuantity = warehousingContent.getTotalQuantity();

                if (totalQuantity != null) {
                    x = x + totalQuantity;
                }

                Integer warehousingQuantity = warehousingContent.getWarehousingQuantity();
                if (warehousingQuantity != null) {
                    y = y + warehousingContent.getWarehousingQuantity();
                }
            }
            warehousingRecordFormVO.setWarehousingQuantity(y);
            warehousingRecordFormVO.setQuantityPurchased(x);
        }
        if (warehousingRecordFormVOList.size() != 0) {

            PageUtils pageUtils = new PageUtils();

            Page pages = pageUtils.getPages((int) page.getCurrent(), (int) page.getSize(), warehousingRecordFormVOList);

            return pages;
        } else return null;
    }

}



